@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 
 # This file is part of ReText
-# Copyright: 2012-2020 Dmitry Shachnev
+# Copyright: 2012-2021 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,18 +19,18 @@
 import sys
 import markups
 import markups.common
-from os.path import dirname, exists, join
+from os.path import dirname, exists, join, expanduser
 
 from PyQt5.QtCore import QByteArray, QLocale, QSettings, QStandardPaths
-from PyQt5.QtGui import QFont
+from PyQt5.QtGui import QFont, QFontDatabase
 
-app_version = "7.1.0"
+app_version = "7.2.0"
 
 settings = QSettings('ReText project', 'ReText')
 
 if not str(settings.fileName()).endswith('.conf'):
 	# We are on Windows probably
-	settings = QSettings(QSettings.IniFormat, QSettings.UserScope,
+	settings = QSettings(QSettings.Format.IniFormat, QSettings.Scope.UserScope,
 		'ReText project', 'ReText')
 
 datadirs = []
@@ -43,7 +43,7 @@ def initializeDataDirs():
 	except NameError:
 		pass
 
-	dataLocations = QStandardPaths.standardLocations(QStandardPaths.GenericDataLocation)
+	dataLocations = QStandardPaths.standardLocations(QStandardPaths.StandardLocation.GenericDataLocation)
 	datadirs.extend(join(d, 'retext') for d in dataLocations)
 
 	if sys.platform == "win32":
@@ -71,18 +71,20 @@ configOptions = {
 	'defaultMarkup': markups.MarkdownMarkup.name,
 	'defaultPreviewState': 'editor',
 	'detectEncoding': True,
+	'directoryPath': expanduser("~"),
 	'documentStatsEnabled': False,
 	'editorFont': QFont(),
 	'font': QFont(),
 	'handleWebLinks': False,
 	'hideToolBar': False,
-	'highlightCurrentLine': False,
+	'highlightCurrentLine': 'disabled',
 	'iconTheme': '',
 	'lastTabIndex': 0,
 	'lineNumbersEnabled': False,
 	'markdownDefaultFileExtension': '.mkd',
 	'openFilesInExistingWindow': True,
 	'openLastFilesOnStartup': False,
+	'orderedListMode': 'increment',
 	'paperSize': '',
 	'pygmentsStyle': 'default',
 	'recentDocumentsCount': 10,
@@ -91,6 +93,7 @@ configOptions = {
 	'rightMargin': 0,
 	'rightMarginWrap': False,
 	'saveWindowGeometry': False,
+	'showDirectoryTree': False,
 	'spellCheck': False,
 	'spellCheckLocale': '',
 	'styleSheet': '',
@@ -154,10 +157,6 @@ def writeListToSettings(key, value, settings=settings):
 def getSettingsFilePath(settings=settings):
 	return settings.fileName()
 
-def chooseMonospaceFont():
-	font = QFont('monospace')
-	font.setStyleHint(QFont.TypeWriter)
-	return font
 
 class ReTextSettings(object):
 	def __init__(self):
@@ -179,7 +178,7 @@ class ReTextSettings(object):
 		if option == 'font' and not value.family():
 			value = QFont()
 		if option == 'editorFont' and not value.family():
-			value = chooseMonospaceFont()
+			value = QFontDatabase.systemFont(QFontDatabase.SystemFont.FixedFont)
 		return value
 
 globalSettings = ReTextSettings()
