@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 #
 # This file is part of ReText
-# Copyright: 2012-2020 Dmitry Shachnev
+# Copyright: 2012-2021 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,8 +38,8 @@ colors = {
 	'currentLineHighlight': QColor(0xff, 0xff, 0xc8),
 	'infoArea': QColor(0xaa, 0xff, 0x55, 0xaa),
 	'statsArea': QColor(0xff, 0xaa, 0x55, 0xaa),
-	'lineNumberArea': Qt.cyan,
-	'lineNumberAreaText': Qt.darkCyan
+	'lineNumberArea': Qt.GlobalColor.cyan,
+	'lineNumberAreaText': Qt.GlobalColor.darkCyan
 }
 
 colorValues = {
@@ -97,8 +97,18 @@ class ReTextEdit(QTextEdit):
 	orderedListPattern = re.compile("^([\\s]*)(\\d+)\\. $")
 	wordPattern = re.compile(r"\w+")
 	nonAlphaNumPattern = re.compile(r"\W")
+	surroundKeysSelfClose = [
+		Qt.Key.Key_Underscore,
+		Qt.Key.Key_Asterisk,
+		Qt.Key.Key_QuoteDbl,
+		Qt.Key.Key_Apostrophe
+	]
+	surroundKeysOtherClose = {
+		Qt.Key.Key_ParenLeft: ')',
+		Qt.Key.Key_BracketLeft: ']'
+	}
 
-	def __init__(self, parent):
+	def __init__(self, parent, settings=globalSettings):
 		QTextEdit.__init__(self)
 		self.tab = weakref.proxy(parent)
 		self.parent = parent.p
@@ -114,22 +124,23 @@ class ReTextEdit(QTextEdit):
 		self.document().blockCountChanged.connect(self.updateLineNumberAreaWidth)
 		self.cursorPositionChanged.connect(self.highlightCurrentLine)
 		self.document().contentsChange.connect(self.contentsChange)
+		self.settings = settings
 		if globalSettings.useFakeVim:
 			self.installFakeVimHandler()
 
 	def setWrapModeAndWidth(self):
 		if globalSettings.rightMarginWrap and (self.rect().topRight().x() > self.marginx):
-			self.setLineWrapMode(QTextEdit.FixedPixelWidth)
+			self.setLineWrapMode(QTextEdit.LineWrapMode.FixedPixelWidth)
 			self.setLineWrapColumnOrWidth(self.marginx)
 		else:
-			self.setLineWrapMode(QTextEdit.WidgetWidth)
+			self.setLineWrapMode(QTextEdit.LineWrapMode.WidgetWidth)
 
 	def updateFont(self):
 		self.setFont(globalSettings.editorFont)
 		metrics = self.fontMetrics()
-		self.marginx = (self.document().documentMargin()
-			+ metrics.width(' ' * globalSettings.rightMargin))
-		self.setTabStopWidth(globalSettings.tabWidth * self.fontMetrics().width(' '))
+		self.marginx = (int(self.document().documentMargin())
+			+ metrics.horizontalAdvance(' ' * globalSettings.rightMargin))
+		self.setTabStopDistance(globalSettings.tabWidth * metrics.horizontalAdvance(' '))
 		self.updateLineNumberAreaWidth()
 		self.infoArea.updateTextAndGeometry()
 		self.updateTextStatistics()
@@ -139,17 +150,17 @@ class ReTextEdit(QTextEdit):
 
 	def paintEvent(self, event):
 		if not globalSettings.rightMargin:
-			return QTextEdit.paintEvent(self, event)
+			return super().paintEvent(event)
 		painter = QPainter(self.viewport())
 		painter.setPen(colorValues['marginLine'])
 		y1 = self.rect().topLeft().y()
 		y2 = self.rect().bottomLeft().y()
 		painter.drawLine(self.marginx, y1, self.marginx, y2)
-		QTextEdit.paintEvent(self, event)
+		super().paintEvent(event)
 
 	def wheelEvent(self, event):
 		modifiers = QGuiApplication.keyboardModifiers()
-		if modifiers == Qt.ControlModifier:
+		if modifiers == Qt.KeyboardModifier.ControlModifier:
 			font = globalSettings.editorFont
 			size = font.pointSize()
 			scroll = event.angleDelta().y()
@@ -162,7 +173,7 @@ class ReTextEdit(QTextEdit):
 			font.setPointSize(size)
 			self.parent.setEditorFont(font)
 		else:
-			QTextEdit.wheelEvent(self, event)
+			super().wheelEvent(event)
 
 			if event.angleDelta().y() < 0:
 				scrollBarLimit = self.verticalScrollBar().maximum()
@@ -173,7 +184,7 @@ class ReTextEdit(QTextEdit):
 				self.scrollLimitReached.emit(event)
 
 	def scrollContentsBy(self, dx, dy):
-		QTextEdit.scrollContentsBy(self, dx, dy)
+		super().scrollContentsBy(dx, dy)
 		self.lineNumberArea.update()
 
 	def contextMenuEvent(self, event):
@@ -198,7 +209,7 @@ class ReTextEdit(QTextEdit):
 		dictionary = self.tab.highlighter.dictionary
 		word = None
 		if isalpha and not (oldcursor.hasSelection() and oldcursor.selectedText() != cursor.selectedText()):
-			cursor.select(QTextCursor.WordUnderCursor)
+			cursor.select(QTextCursor.SelectionType.WordUnderCursor)
 			word = cursor.selectedText()
 
 		if word is not None and dictionary and not dictionary.check(word):
@@ -234,41 +245,60 @@ class ReTextEdit(QTextEdit):
 		dictionary.add(newword)
 		self.tab.highlighter.rehighlightBlock(block)
 
+	def isSurroundKey(self, key):
+		return key in self.surroundKeysSelfClose or key in self.surroundKeysOtherClose
+
+	def getCloseKey(self, event, key):
+		if key in self.surroundKeysSelfClose:
+			return event.text()
+
+		if key in self.surroundKeysOtherClose:
+			return self.surroundKeysOtherClose[key]
+
+	def surroundText(self, cursor, event, key):
+		text = cursor.selectedText()
+		keyStr = event.text()
+		keyClose = self.getCloseKey(event, key)
+
+		cursor.insertText(keyStr + text + keyClose)
+
 	def keyPressEvent(self, event):
 		key = event.key()
 		cursor = self.textCursor()
-		if key == Qt.Key_Backspace and event.modifiers() & Qt.GroupSwitchModifier:
+		if key == Qt.Key.Key_Backspace and event.modifiers() & Qt.KeyboardModifier.GroupSwitchModifier:
 			# Workaround for https://bugreports.qt.io/browse/QTBUG-49771
 			event = QKeyEvent(event.type(), event.key(),
-				event.modifiers() ^ Qt.GroupSwitchModifier)
-		if key == Qt.Key_Tab:
+				event.modifiers() ^ Qt.KeyboardModifier.GroupSwitchModifier)
+		if key == Qt.Key.Key_Tab:
 			documentIndentMore(self.document(), cursor)
-		elif key == Qt.Key_Backtab:
+		elif key == Qt.Key.Key_Backtab:
 			documentIndentLess(self.document(), cursor)
-		elif key == Qt.Key_Return:
+		elif key == Qt.Key.Key_Return:
 			markupClass = self.tab.getActiveMarkupClass()
-			if event.modifiers() & Qt.ControlModifier:
+			if event.modifiers() & Qt.KeyboardModifier.ControlModifier:
 				cursor.insertText('\n')
 				self.ensureCursorVisible()
 			elif self.tableModeEnabled and tablemode.handleReturn(cursor, markupClass,
-					newRow=(event.modifiers() & Qt.ShiftModifier)):
+					newRow=(event.modifiers() & Qt.KeyboardModifier.ShiftModifier)):
 				self.setTextCursor(cursor)
 				self.ensureCursorVisible()
 			else:
-				if event.modifiers() & Qt.ShiftModifier and markupClass == MarkdownMarkup:
+				if event.modifiers() & Qt.KeyboardModifier.ShiftModifier and markupClass == MarkdownMarkup:
 					# Insert Markdown-style line break
 					cursor.insertText('  ')
 				self.handleReturn(cursor)
+		elif cursor.selectedText() and self.isSurroundKey(key):
+			self.surroundText(cursor, event, key)
 		else:
 			if event.text() and self.tableModeEnabled:
 				cursor.beginEditBlock()
-			QTextEdit.keyPressEvent(self, event)
+			super().keyPressEvent(event)
 			if event.text() and self.tableModeEnabled:
 				cursor.endEditBlock()
 
 	def handleReturn(self, cursor):
 		# Select text between the cursor and the line start
-		cursor.movePosition(QTextCursor.StartOfBlock, QTextCursor.KeepAnchor)
+		cursor.movePosition(QTextCursor.MoveOperation.StartOfBlock, QTextCursor.MoveMode.KeepAnchor)
 		text = cursor.selectedText()
 		length = len(text)
 		match = self.returnBlockPattern.search(text)
@@ -282,7 +312,8 @@ class ReTextEdit(QTextEdit):
 				if matchOL is not None:
 					matchedPrefix = matchOL.group(1)
 					matchedNumber = int(matchOL.group(2))
-					matchedText = matchedPrefix + str(matchedNumber + 1) + ". "
+					nextNumber = matchedNumber if self.settings.orderedListMode == 'repeat' else matchedNumber + 1
+					matchedText = matchedPrefix + str(nextNumber) + ". "
 		else:
 			matchedText = ''
 		# Reset the cursor
@@ -291,25 +322,25 @@ class ReTextEdit(QTextEdit):
 		self.ensureCursorVisible()
 
 	def moveLineUp(self):
-		self.moveLine(QTextCursor.PreviousBlock)
+		self.moveLine(QTextCursor.MoveOperation.PreviousBlock)
 
 	def moveLineDown(self):
-		self.moveLine(QTextCursor.NextBlock)
+		self.moveLine(QTextCursor.MoveOperation.NextBlock)
 
 	def moveLine(self, direction):
 		cursor = self.textCursor()
 		# Select the current block
-		cursor.movePosition(QTextCursor.StartOfBlock, QTextCursor.MoveAnchor)
-		cursor.movePosition(QTextCursor.NextBlock, QTextCursor.KeepAnchor)
+		cursor.movePosition(QTextCursor.MoveOperation.StartOfBlock, QTextCursor.MoveMode.MoveAnchor)
+		cursor.movePosition(QTextCursor.MoveOperation.NextBlock, QTextCursor.MoveMode.KeepAnchor)
 		text = cursor.selectedText()
 		# Remove it
 		cursor.removeSelectedText()
 		# Move to the wanted block
-		cursor.movePosition(direction, QTextCursor.MoveAnchor)
+		cursor.movePosition(direction, QTextCursor.MoveMode.MoveAnchor)
 		# Paste the line
 		cursor.insertText(text)
 		# Move to the pasted block
-		cursor.movePosition(QTextCursor.PreviousBlock, QTextCursor.MoveAnchor)
+		cursor.movePosition(QTextCursor.MoveOperation.PreviousBlock, QTextCursor.MoveMode.MoveAnchor)
 		# Update cursor
 		self.setTextCursor(cursor)
 
@@ -317,18 +348,18 @@ class ReTextEdit(QTextEdit):
 		if not globalSettings.lineNumbersEnabled:
 			return 0
 		cursor = QTextCursor(self.document())
-		cursor.movePosition(QTextCursor.End)
+		cursor.movePosition(QTextCursor.MoveOperation.End)
 		if globalSettings.relativeLineNumbers:
 			digits = len(str(cursor.blockNumber())) + 1
 		else:
 			digits = len(str(cursor.blockNumber() + 1))
-		return 5 + self.fontMetrics().width('9') * digits
+		return 5 + self.fontMetrics().horizontalAdvance('9') * digits
 
 	def updateLineNumberAreaWidth(self, blockcount=0):
 		self.setViewportMargins(self.lineNumberAreaWidth(), 0, 0, 0)
 
 	def resizeEvent(self, event):
-		QTextEdit.resizeEvent(self, event)
+		super().resizeEvent(event)
 		rect = self.contentsRect()
 		self.resized.emit(rect)
 		self.lineNumberArea.setGeometry(rect.left(), rect.top(),
@@ -341,14 +372,27 @@ class ReTextEdit(QTextEdit):
 	def highlightCurrentLine(self):
 		if globalSettings.relativeLineNumbers:
 			self.lineNumberArea.update()
-		if not globalSettings.highlightCurrentLine:
+		if globalSettings.highlightCurrentLine == 'disabled':
 			return self.setExtraSelections([])
-		selection = QTextEdit.ExtraSelection();
+		selection = QTextEdit.ExtraSelection()
 		selection.format.setBackground(colorValues['currentLineHighlight'])
-		selection.format.setProperty(QTextFormat.FullWidthSelection, True)
+		selection.format.setProperty(QTextFormat.Property.FullWidthSelection, True)
 		selection.cursor = self.textCursor()
 		selection.cursor.clearSelection()
-		self.setExtraSelections([selection])
+		selections = [selection]
+		if globalSettings.highlightCurrentLine == 'wrapped-line':
+			selections.append(QTextEdit.ExtraSelection())
+			selections[0].cursor.movePosition(QTextCursor.MoveOperation.StartOfBlock)
+			selections[0].cursor.movePosition(QTextCursor.MoveOperation.EndOfBlock, QTextCursor.MoveMode.KeepAnchor)
+			selections[1].format.setBackground(colorValues['currentLineHighlight'])
+			selections[1].format.setProperty(QTextFormat.Property.FullWidthSelection, True)
+			selections[1].cursor = self.textCursor()
+			selections[1].cursor.movePosition(QTextCursor.MoveOperation.EndOfBlock)
+		elif selection.cursor.block().textDirection() == Qt.LayoutDirection.RightToLeft:
+			# FullWidthSelection does not work correctly for RTL direction
+			selection.cursor.movePosition(QTextCursor.MoveOperation.StartOfLine)
+			selection.cursor.movePosition(QTextCursor.MoveOperation.EndOfLine, QTextCursor.MoveMode.KeepAnchor)
+		self.setExtraSelections(selections)
 
 	def enableTableMode(self, enable):
 		self.tableModeEnabled = enable
@@ -460,7 +504,7 @@ class LineNumberArea(QWidget):
 
 	def paintEvent(self, event):
 		if not globalSettings.lineNumbersEnabled:
-			return QWidget.paintEvent(self, event)
+			return super().paintEvent(event)
 		painter = QPainter(self)
 		painter.fillRect(event.rect(), colorValues['lineNumberArea'])
 		painter.setPen(colorValues['lineNumberAreaText'])
@@ -478,11 +522,11 @@ class LineNumberArea(QWidget):
 				break
 			number = str(cursor.blockNumber() - relativeTo).replace('-', '−')
 			painter.drawText(0, rect.top(), self.width() - 2,
-			                 fontHeight, Qt.AlignRight, number)
-			cursor.movePosition(QTextCursor.EndOfBlock)
+			                 fontHeight, Qt.AlignmentFlag.AlignRight, number)
+			cursor.movePosition(QTextCursor.MoveOperation.EndOfBlock)
 			atEnd = cursor.atEnd()
 			if not atEnd:
-				cursor.movePosition(QTextCursor.NextBlock)
+				cursor.movePosition(QTextCursor.MoveOperation.NextBlock)
 
 class InfoArea(QLabel):
 	def __init__(self, editor, baseColor):
@@ -493,9 +537,9 @@ class InfoArea(QLabel):
 		self.setAutoFillBackground(True)
 		self.baseColor = baseColor
 		palette = self.palette()
-		palette.setColor(QPalette.Window, self.baseColor)
+		palette.setColor(QPalette.ColorRole.Window, self.baseColor)
 		self.setPalette(palette)
-		self.setCursor(Qt.IBeamCursor)
+		self.setCursor(Qt.CursorShape.IBeamCursor)
 
 	def updateTextAndGeometry(self):
 		text = self.getText()
@@ -508,7 +552,7 @@ class InfoArea(QLabel):
 
 	def getAreaSize(self, text):
 		metrics = self.fontMetrics()
-		width = metrics.width(text)
+		width = metrics.horizontalAdvance(text)
 		height = metrics.height()
 		return width, height
 
@@ -522,17 +566,17 @@ class InfoArea(QLabel):
 		palette = self.palette()
 		windowColor = QColor(self.baseColor)
 		windowColor.setAlpha(0x20)
-		palette.setColor(QPalette.Window, windowColor)
-		textColor = palette.color(QPalette.WindowText)
+		palette.setColor(QPalette.ColorRole.Window, windowColor)
+		textColor = palette.color(QPalette.ColorRole.WindowText)
 		textColor.setAlpha(0x20)
-		palette.setColor(QPalette.WindowText, textColor)
+		palette.setColor(QPalette.ColorRole.WindowText, textColor)
 		self.setPalette(palette)
 
 	def leaveEvent(self, event):
 		palette = self.palette()
-		palette.setColor(QPalette.Window, self.baseColor)
-		palette.setColor(QPalette.WindowText,
-			self.editor.palette().color(QPalette.WindowText))
+		palette.setColor(QPalette.ColorRole.Window, self.baseColor)
+		palette.setColor(QPalette.ColorRole.WindowText,
+			self.editor.palette().color(QPalette.ColorRole.WindowText))
 		self.setPalette(palette)
 
 	def mousePressEvent(self, event):
