@@ -1,5 +1,5 @@
 # This file is part of ReText
-# Copyright: 2012-2015 Dmitry Shachnev
+# Copyright: 2012-2021 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -15,6 +15,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 from ReText import settings
+from enum import IntFlag, auto
 import re
 
 from PyQt5.QtCore import Qt
@@ -44,19 +45,19 @@ reTextileQuot  = re.compile(r'^bq\.\s.+')
 reMkdCodeSpans = re.compile('`[^`]*`')
 reMkdMathSpans = re.compile(r'\\[\(\[].*?\\[\)\]]')
 reReSTCodeSpan = re.compile('``.+?``')
-reWords        = re.compile('[^_\\W]+', flags=re.UNICODE)
-reSpacesOnEnd  = re.compile(r'\s+$', flags=re.UNICODE)
+reWords        = re.compile('[^_\\W]+')
+reSpacesOnEnd  = re.compile(r'\s+$')
 
 defaultColorScheme = {
-	'htmlTags': Qt.darkMagenta,
-	'htmlSymbols': Qt.darkCyan,
-	'htmlStrings': Qt.darkYellow,
-	'htmlComments': Qt.gray,
+	'htmlTags': Qt.GlobalColor.darkMagenta,
+	'htmlSymbols': Qt.GlobalColor.darkCyan,
+	'htmlStrings': Qt.GlobalColor.darkYellow,
+	'htmlComments': Qt.GlobalColor.gray,
 	'codeSpans': QColor(0x50, 0x50, 0x50),
 	'markdownLinks': QColor(0, 0, 0x90),
-	'blockquotes': Qt.darkGray,
-	'restDirectives': Qt.darkMagenta,
-	'restRoles': Qt.darkRed,
+	'blockquotes': Qt.GlobalColor.darkGray,
+	'restDirectives': Qt.GlobalColor.darkMagenta,
+	'restRoles': Qt.GlobalColor.darkRed,
 	'whitespaceOnEnd': QColor(0xe1, 0xe1, 0xa5, 0x80)
 }
 colorScheme = {}
@@ -103,54 +104,77 @@ def QString_length(text):
 	# in it (which is the case with Python strings).
 	return sum(2 if ord(char) > 65535 else 1 for char in text)
 
+
+class Markup(IntFlag):
+	Mkd = auto()
+	ReST = auto()
+	Textile = auto()
+	HTML = auto()
+
+	# Special value which means that no other markup is allowed inside this pattern
+	CodeSpan = auto()
+
+
+docTypesMapping = {
+	'Markdown': Markup.Mkd,
+	'reStructuredText': Markup.ReST,
+	'Textile': Markup.Textile,
+	'html': Markup.HTML,
+}
+
+
 class ReTextHighlighter(QSyntaxHighlighter):
 	dictionary = None
 	docType = None
 
 	patterns = (
-		# regex,         color,
-		(reHtmlTags,     FG('htmlTags') | QFont.Bold),                          # 0
-		(reHtmlSymbols,  FG('htmlSymbols') | QFont.Bold),                       # 1
-		(reHtmlStrings,  FG('htmlStrings') | QFont.Bold),                       # 2
-		(reHtmlComments, FG('htmlComments')),                                   # 3
-		(reAsterisks,    ITAL),                                                 # 4
-		(reUnderline,    ITAL),                                                 # 5
-		(reDblAsterisks, NF | QFont.Bold),                                      # 6
-		(reDblUnderline, NF | QFont.Bold),                                      # 7
-		(reTrpAsterisks, ITAL | QFont.Bold),                                    # 8
-		(reTrpUnderline, ITAL | QFont.Bold),                                    # 9
-		(reMkdHeaders,   NF | QFont.Black),                                     # 10
-		(reMkdLinksImgs, FG('markdownLinks')),                                  # 11
-		(reMkdLinkRefs,  ITAL | UNDL),                                          # 12
-		(reBlockQuotes,  FG('blockquotes')),                                    # 13
-		(reReSTDirects,  FG('restDirectives') | QFont.Bold),                    # 14
-		(reReSTRoles,    NF, FG('restRoles') | QFont.Bold, FG('htmlStrings')),  # 15
-		(reTextileHdrs,  NF | QFont.Black),                                     # 16
-		(reTextileQuot,  FG('blockquotes')),                                    # 17
-		(reAsterisks,    NF | QFont.Bold),                                      # 18
-		(reDblUnderline, ITAL),                                                 # 19
-		(reMkdCodeSpans, FG('codeSpans')),                                      # 20
-		(reReSTCodeSpan, FG('codeSpans')),                                      # 21
-		(reReSTLinks,    NF, NF, ITAL | UNDL, NF),                              # 22
-		(reReSTLinkRefs, NF, FG('markdownLinks'), ITAL | UNDL),                 # 23
-		(reReSTFldLists, NF, FG('restDirectives')),                             # 24
-		(reMkdMathSpans, FG('codeSpans')),                                      # 25
+		# regex,         color,                                markups
+		(reMkdCodeSpans, FG('codeSpans'),                      Markup.Mkd | Markup.CodeSpan),
+		(reMkdMathSpans, FG('codeSpans'),                      Markup.Mkd | Markup.CodeSpan),
+		(reReSTCodeSpan, FG('codeSpans'),                      Markup.ReST | Markup.CodeSpan),
+		(reHtmlTags,     FG('htmlTags') | QFont.Weight.Bold,   Markup.Mkd | Markup.Textile | Markup.HTML),
+		(reHtmlSymbols,  FG('htmlSymbols') | QFont.Weight.Bold, Markup.Mkd | Markup.HTML),
+		(reHtmlStrings,  FG('htmlStrings') | QFont.Weight.Bold, Markup.Mkd | Markup.HTML),
+		(reHtmlComments, FG('htmlComments'),                   Markup.Mkd | Markup.HTML),
+		(reAsterisks,    ITAL,                                 Markup.Mkd | Markup.ReST),
+		(reUnderline,    ITAL,                                 Markup.Mkd | Markup.Textile),
+		(reDblAsterisks, NF | QFont.Weight.Bold,               Markup.Mkd | Markup.ReST | Markup.Textile),
+		(reDblUnderline, NF | QFont.Weight.Bold,               Markup.Mkd),
+		(reTrpAsterisks, ITAL | QFont.Weight.Bold,             Markup.Mkd),
+		(reTrpUnderline, ITAL | QFont.Weight.Bold,             Markup.Mkd),
+		(reMkdHeaders,   NF | QFont.Weight.Black,              Markup.Mkd),
+		(reMkdLinksImgs, FG('markdownLinks'),                  Markup.Mkd),
+		(reMkdLinkRefs,  ITAL | UNDL,                          Markup.Mkd),
+		(reBlockQuotes,  FG('blockquotes'),                    Markup.Mkd),
+		(reReSTDirects,  FG('restDirectives') | QFont.Weight.Bold, Markup.ReST),
+		(reReSTRoles,    NF, FG('restRoles') | QFont.Weight.Bold, FG('htmlStrings'), Markup.ReST),
+		(reTextileHdrs,  NF | QFont.Weight.Black,              Markup.Textile),
+		(reTextileQuot,  FG('blockquotes'),                    Markup.Textile),
+		(reAsterisks,    NF | QFont.Weight.Bold,               Markup.Textile),
+		(reDblUnderline, ITAL,                                 Markup.Textile),
+		(reReSTLinks,    NF, NF, ITAL | UNDL, NF,              Markup.ReST),
+		(reReSTLinkRefs, NF, FG('markdownLinks'), ITAL | UNDL, Markup.ReST),
+		(reReSTFldLists, NF, FG('restDirectives'),             Markup.ReST),
 	)
 
-	patternsDict = {
-		'Markdown': (0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 20, 25),
-		'reStructuredText': (4, 6, 14, 15, 21, 22, 23, 24),
-		'Textile': (0, 5, 6, 16, 17, 18, 19),
-		'html': (0, 1, 2, 3)
-	}
-
 	def highlightBlock(self, text):
 		# Syntax highlighter
-		if self.docType in self.patternsDict:
-			for number in self.patternsDict[self.docType]:
-				pattern = self.patterns[number]
-				for match in pattern[0].finditer(text):
-					for i, formatter in enumerate(pattern[1:]):
+		codeSpans = set()
+		if self.docType in docTypesMapping:
+			markup = docTypesMapping[self.docType]
+			for pattern, *formatters, markups in self.patterns:
+				if not (markup & markups):
+					continue
+				for match in pattern.finditer(text):
+					start, end = match.start(), match.end()
+					if markups & Markup.CodeSpan:
+						codeSpans.add((start, end))
+					elif any(start < codeEnd and end > codeStart
+					         for codeStart, codeEnd in codeSpans):
+						# Ignore any syntax if its match intersects with code spans.
+						# See https://github.com/retext-project/retext/issues/529
+						continue
+					for i, formatter in enumerate(formatters):
 						charFormat = QTextCharFormat()
 						formatter.format(charFormat)
 						self.setFormat(QString_length(text[:match.start(i)]),
@@ -165,8 +189,8 @@ class ReTextHighlighter(QSyntaxHighlighter):
 		# Spell checker
 		if self.dictionary:
 			charFormat = QTextCharFormat()
-			charFormat.setUnderlineColor(Qt.red)
-			charFormat.setUnderlineStyle(QTextCharFormat.SpellCheckUnderline)
+			charFormat.setUnderlineColor(Qt.GlobalColor.red)
+			charFormat.setUnderlineStyle(QTextCharFormat.UnderlineStyle.SpellCheckUnderline)
 			for match in reWords.finditer(text):
 				finalFormat = QTextCharFormat()
 				finalFormat.merge(charFormat)
