@@ -1,7 +1,7 @@
 # vim: ts=8:sts=8:sw=8:noexpandtab
 #
 # This file is part of ReText
-# Copyright: 2015-2020 Dmitry Shachnev
+# Copyright: 2015-2021 Dmitry Shachnev
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ class ReTextTab(QSplitter):
 		return self._fileName
 
 	def __init__(self, parent, fileName, previewState=PreviewDisabled):
-		super().__init__(Qt.Horizontal, parent=parent)
+		super().__init__(Qt.Orientation.Horizontal, parent=parent)
 		self.p = parent
 		self._fileName = fileName
 		self.editBox = ReTextEdit(self)
@@ -206,7 +206,7 @@ class ReTextTab(QSplitter):
 			if globalSettings.useWebKit:
 				# https://github.com/retext-project/retext/pull/187
 				palette = QApplication.palette()
-				style += '@media screen { html { color: %s; } }\n' % palette.color(QPalette.WindowText).name()
+				style += '@media screen { html { color: %s; } }\n' % palette.color(QPalette.ColorRole.WindowText).name()
 				# https://github.com/retext-project/retext/issues/408
 				style += '@media print { html { background-color: white; } }\n'
 			headers += '<style type="text/css">\n' + style + '</style>\n'
@@ -275,6 +275,8 @@ class ReTextTab(QSplitter):
 
 	def triggerPreviewUpdate(self):
 		self.previewOutdated = True
+		if self.previewState == PreviewDisabled:
+			return
 
 		if not self.conversionPending:
 			self.conversionPending = True
@@ -340,7 +342,7 @@ class ReTextTab(QSplitter):
 		# TODO: why do we open the file twice: for detecting encoding
 		# and for actual read? Can we open it just once?
 		openfile = QFile(self._fileName)
-		openfile.open(QFile.ReadOnly)
+		openfile.open(QFile.OpenModeFlag.ReadOnly)
 		stream = QTextStream(openfile)
 		encoding = encoding or globalSettings.defaultCodec
 		if encoding:
@@ -372,11 +374,11 @@ class ReTextTab(QSplitter):
 		# see https://bugreports.qt.io/browse/QTBUG-57552 and
 		# https://github.com/retext-project/retext/issues/216
 		cursor = self.editBox.textCursor()
-		cursor.select(QTextCursor.Document)
+		cursor.select(QTextCursor.SelectionType.Document)
 		text = cursor.selectedText().replace('\u2029', '\n')
 
 		savefile = QFile(fileName or self._fileName)
-		result = savefile.open(QFile.WriteOnly)
+		result = savefile.open(QFile.OpenModeFlag.WriteOnly)
 		if result:
 			savestream = QTextStream(savefile)
 
@@ -411,11 +413,13 @@ class ReTextTab(QSplitter):
 			self.editBox.setTextCursor(newCursor)
 
 	def find(self, text, flags, replaceText=None, wrap=False):
+		if self.previewState == PreviewNormal and replaceText is None:
+			return self.previewBox.findText(text, flags)
 		cursor = self.editBox.textCursor()
-		if wrap and flags & QTextDocument.FindBackward:
-			cursor.movePosition(QTextCursor.End)
+		if wrap and flags & QTextDocument.FindFlag.FindBackward:
+			cursor.movePosition(QTextCursor.MoveOperation.End)
 		elif wrap:
-			cursor.movePosition(QTextCursor.Start)
+			cursor.movePosition(QTextCursor.MoveOperation.Start)
 		if replaceText is not None and cursor.selectedText() == text:
 			newCursor = cursor
 		else:
@@ -423,8 +427,8 @@ class ReTextTab(QSplitter):
 		if not newCursor.isNull():
 			if replaceText is not None:
 				newCursor.insertText(replaceText)
-				newCursor.movePosition(QTextCursor.Left, QTextCursor.MoveAnchor, len(replaceText))
-				newCursor.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, len(replaceText))
+				newCursor.movePosition(QTextCursor.MoveOperation.Left, QTextCursor.MoveMode.MoveAnchor, len(replaceText))
+				newCursor.movePosition(QTextCursor.MoveOperation.Right, QTextCursor.MoveMode.KeepAnchor, len(replaceText))
 			self.editBox.setTextCursor(newCursor)
 			if self.editBox.cursorRect().bottom() >= self.editBox.height() - 3:
 				scrollValue = self.editBox.verticalScrollBar().value()
@@ -438,7 +442,7 @@ class ReTextTab(QSplitter):
 	def replaceAll(self, text, replaceText):
 		cursor = self.editBox.textCursor()
 		cursor.beginEditBlock()
-		cursor.movePosition(QTextCursor.Start)
+		cursor.movePosition(QTextCursor.MoveOperation.Start)
 		flags = QTextDocument.FindFlags()
 		cursor = lastCursor = self.editBox.document().find(text, cursor, flags)
 		while not cursor.isNull():
@@ -446,8 +450,8 @@ class ReTextTab(QSplitter):
 			lastCursor = cursor
 			cursor = self.editBox.document().find(text, cursor, flags)
 		if not lastCursor.isNull():
-			lastCursor.movePosition(QTextCursor.Left, QTextCursor.MoveAnchor, len(replaceText))
-			lastCursor.movePosition(QTextCursor.Right, QTextCursor.KeepAnchor, len(replaceText))
+			lastCursor.movePosition(QTextCursor.MoveOperation.Left, QTextCursor.MoveMode.MoveAnchor, len(replaceText))
+			lastCursor.movePosition(QTextCursor.MoveOperation.Right, QTextCursor.MoveMode.KeepAnchor, len(replaceText))
 			self.editBox.setTextCursor(lastCursor)
 		self.editBox.textCursor().endEditBlock()
 		return not lastCursor.isNull()
@@ -477,11 +481,11 @@ class ReTextTab(QSplitter):
 		"""
 		buttonReply = QMessageBox.question(self, self.tr('Create missing file?'),
 		                                   self.tr("The file '%s' does not exist.\n\nDo you want to create it?") % fileToCreate,
-		                                   QMessageBox.Yes | QMessageBox.No,
-		                                   QMessageBox.No)
-		if buttonReply == QMessageBox.Yes:
+		                                   QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No,
+		                                   QMessageBox.StandardButton.No)
+		if buttonReply == QMessageBox.StandardButton.Yes:
 			return self.createFile(fileToCreate)
-		elif buttonReply == QMessageBox.No:
+		elif buttonReply == QMessageBox.StandardButton.No:
 			return False
 
 	def resolveSourceFile(self, linkPath):
